{-# OPTIONS -cpp -fffi -fglasgow-exts #-}

-- Module:    Data.ByteString.ASCII

-- Manipulate ByteStrings using Char operations, interpreting the strings
-- as ASCII, including C0 Controls.  This is similar to Char8, but words > 127 -- will be tucked safely away, out of reach for isUpper and friends.
-- (We use the "private" codepoints 0xF000..0xF07F for this purpose)

module Data.ByteString.ASCII
#include "char_functions.hsi"

charset_name :: String
charset_name = "ASCII"

charset = [0..127]++[0xF000..0xF07F]

is_valid :: Char -> Bool
is_valid c = basic c || extended c

basic :: Char -> Bool
basic c = ord c < 128 

extended :: Char -> Bool
extended c = ord c >= 0xF000 && ord c <= 0xF07F

base = 0xF000-0x0080

-- | Conversion between 'Char' and 'Word8'.
c2w :: Char -> Word8
c2w c = if basic c then c2w_raw c
        else if extended c then (fromIntegral . (\x-> x+base) . ord) c
        else err_oor c
{-# INLINE c2w #-}

w2c :: Word8 -> Char
w2c w = if w < 128 then w2c_raw w else chr (fromIntegral w+base)
{-# INLINE w2c #-}

space, newline :: Word8
space = 0x20
newline = 0x0A

isSpaceWord8 :: Word8 -> Bool
isSpaceWord8 = isAsciiSpace
{-# INLINE isSpaceWord8 #-}
