{-# OPTIONS -cpp -fffi -fglasgow-exts #-}
--
-- Module      : Data.ByteString.Char8
-- Copyright   : (c) Don Stewart 2006
-- License     : BSD-style
--
-- Maintainer  : dons@cse.unsw.edu.au
-- Stability   : experimental
-- Portability : portable (tested with GHC>=6.4.1 and Hugs 2005)
-- 

--
-- | Manipulate ByteStrings using Char operations. All Chars will be
-- truncated to 8 bits. It can be expected that these functions will run
-- at identical speeds to their Word8 equivalents in @Data.ByteString@.
--
-- More specifically these byte strings are taken to be in the
-- subset of Unicode covered by code points 0-255. This covers
-- Unicode Basic Latin, Latin-1 Supplement and C0+C1 Controls.
-- 
-- See: 
--
--  * <http://www.unicode.org/charts/>
--
--  * <http://www.unicode.org/charts/PDF/U0000.pdf>
--
--  * <http://www.unicode.org/charts/PDF/U0080.pdf>
--
-- This module is intended to be imported @qualified@, to avoid name
-- clashes with Prelude functions.  eg.
--
-- > import qualified Data.ByteString.Char8 as B
--

module Data.ByteString.Char8 
#include "char_functions.hsi"

charset_name = "raw"

charset = [0..255]

isSpaceWord8 :: Word8 -> Bool
STRICT1(isSpaceWord8)
isSpaceWord8 w = isAsciiSpace w || w == 0xA0

space, newline :: Word8
space = 0x20
newline = 0x0A

is_valid :: Char -> Bool
is_valid _ = True

c2w :: Char -> Word8
c2w = c2w_raw

w2c :: Word8 -> Char
w2c = w2c_raw




