{-# OPTIONS -cpp -fffi -fglasgow-exts #-}

-- Module:    Data.ByteString.ISO8859_16

-- Manipulate ByteStrings using Char operations, interpreting the strings
-- as ISO8859_16, a.k.a ISO-IR-226, a.k.a. Latin-10. 

module Data.ByteString.ISO8859_16
#include "char_functions.hsi"

charset_name :: String
charset_name = "ISO8859_16"

space, newline :: Word8
space = 0x20
newline = 0x0A

isSpaceWord8 :: Word8 -> Bool
isSpaceWord8 w = case w of 0xA0 -> True -- NO-BREAK SPACE (ISO 8859-1)
                           _   -> isAsciiSpace w
{-# INLINE isSpaceWord8 #-}

#include "derive.hsi"

is_valid :: Char -> Bool
STRICT1(is_valid)
is_valid c = ord c < 0xA0
          || (case bsearch char_idx 0xA0 0xFF c of Just _  -> True; Nothing -> False)

c2w :: Char -> Word8
STRICT1(c2w)
c2w c    = if ord c <= 0x9F then c2w_raw c else c2w_d c

w2c :: Word8 -> Char
STRICT1(w2c)
w2c w    = if w < 0xA0 then w2c_raw w else w2c_d w

charset :: [Int]
charset = [0..0x9F]   -- ASCII subset + controls
       ++ [0x00A0 --   NO-BREAK SPACE, 0xA0
          ,0x0104 --   LATIN CAPITAL LETTER A WITH OGONEK, 0xA1
          ,0x0105 --   LATIN SMALL LETTER A WITH OGONEK, 0xA2
          ,0x0141 --   LATIN CAPITAL LETTER L WITH STROKE, 0xA3
          ,0x20AC --   EURO SIGN, 0xA4
          ,0x00AB --   LEFT-POINTING DOUBLE ANGLE QUOTATION MARK, 0xA5
          ,0x0160 --   LATIN CAPITAL LETTER S WITH CARON, 0xA6
          ,0x00A7 --   SECTION SIGN, 0xA7
          ,0x0161 --   LATIN SMALL LETTER S WITH CARON, 0xA8
          ,0x00A9 --   COPYRIGHT SIGN, 0xA9
          ,0x0218 --   LATIN CAPITAL LETTER S WITH COMMA BELOW, 0xAA
          ,0x201E --   DOUBLE LOW-9 QUOTATION MARK, 0xAB
          ,0x0179 --   LATIN CAPITAL LETTER Z WITH ACUTE, 0xAC
          ,0x00AD --   SOFT HYPHEN, 0xAD
          ,0x017A --   LATIN SMALL LETTER Z WITH ACUTE, 0xAE
          ,0x017B --   LATIN CAPITAL LETTER Z WITH DOT ABOVE, 0xAF
          ,0x00B0 --   DEGREE SIGN, 0xB0
          ,0x00B1 --   PLUS-MINUS SIGN, 0xB1
          ,0x010C --   LATIN CAPITAL LETTER C WITH CARON, 0xB2
          ,0x0142 --   LATIN SMALL LETTER L WITH STROKE, 0xB3
          ,0x017D --   LATIN CAPITAL LETTER Z WITH CARON, 0xB4
          ,0x201D --   RIGHT DOUBLE QUOTATION MARK, 0xB5
          ,0x00B6 --   PILCROW SIGN, 0xB6
          ,0x00B7 --   MIDDLE DOT, 0xB7
          ,0x017E --   LATIN SMALL LETTER Z WITH CARON, 0xB8
          ,0x010D --   LATIN SMALL LETTER C WITH CARON, 0xB9
          ,0x0219 --   LATIN SMALL LETTER S WITH COMMA BELOW, 0xBA
          ,0x00BB --   RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK, 0xBB
          ,0x0152 --   LATIN CAPITAL LIGATURE OE, 0xBC
          ,0x0153 --   LATIN SMALL LIGATURE OE, 0xBD
          ,0x0178 --   LATIN CAPITAL LETTER Y WITH DIAERESIS, 0xBE
          ,0x017C --   LATIN SMALL LETTER Z WITH DOT ABOVE, 0xBF
          ,0x00C0 --   LATIN CAPITAL LETTER A WITH GRAVE, 0xC0
          ,0x00C1 --   LATIN CAPITAL LETTER A WITH ACUTE, 0xC1
          ,0x00C2 --   LATIN CAPITAL LETTER A WITH CIRCUMFLEX, 0xC2
          ,0x0102 --   LATIN CAPITAL LETTER A WITH BREVE, 0xC3
          ,0x00C4 --   LATIN CAPITAL LETTER A WITH DIAERESIS, 0xC4
          ,0x0106 --   LATIN CAPITAL LETTER C WITH ACUTE, 0xC5
          ,0x00C6 --   LATIN CAPITAL LETTER AE, 0xC6
          ,0x00C7 --   LATIN CAPITAL LETTER C WITH CEDILLA, 0xC7
          ,0x00C8 --   LATIN CAPITAL LETTER E WITH GRAVE, 0xC8
          ,0x00C9 --   LATIN CAPITAL LETTER E WITH ACUTE, 0xC9
          ,0x00CA --   LATIN CAPITAL LETTER E WITH CIRCUMFLEX, 0xCA
          ,0x00CB --   LATIN CAPITAL LETTER E WITH DIAERESIS, 0xCB
          ,0x00CC --   LATIN CAPITAL LETTER I WITH GRAVE, 0xCC
          ,0x00CD --   LATIN CAPITAL LETTER I WITH ACUTE, 0xCD
          ,0x00CE --   LATIN CAPITAL LETTER I WITH CIRCUMFLEX, 0xCE
          ,0x00CF --   LATIN CAPITAL LETTER I WITH DIAERESIS, 0xCF
          ,0x0110 --   LATIN CAPITAL LETTER D WITH STROKE, 0xD0
          ,0x0143 --   LATIN CAPITAL LETTER N WITH ACUTE, 0xD1
          ,0x00D2 --   LATIN CAPITAL LETTER O WITH GRAVE, 0xD2
          ,0x00D3 --   LATIN CAPITAL LETTER O WITH ACUTE, 0xD3
          ,0x00D4 --   LATIN CAPITAL LETTER O WITH CIRCUMFLEX, 0xD4
          ,0x0150 --   LATIN CAPITAL LETTER O WITH DOUBLE ACUTE, 0xD5
          ,0x00D6 --   LATIN CAPITAL LETTER O WITH DIAERESIS, 0xD6
          ,0x015A --   LATIN CAPITAL LETTER S WITH ACUTE, 0xD7
          ,0x0170 --   LATIN CAPITAL LETTER U WITH DOUBLE ACUTE, 0xD8
          ,0x00D9 --   LATIN CAPITAL LETTER U WITH GRAVE, 0xD9
          ,0x00DA --   LATIN CAPITAL LETTER U WITH ACUTE, 0xDA
          ,0x00DB --   LATIN CAPITAL LETTER U WITH CIRCUMFLEX, 0xDB
          ,0x00DC --   LATIN CAPITAL LETTER U WITH DIAERESIS, 0xDC
          ,0x0118 --   LATIN CAPITAL LETTER E WITH OGONEK, 0xDD
          ,0x021A --   LATIN CAPITAL LETTER T WITH COMMA BELOW, 0xDE
          ,0x00DF --   LATIN SMALL LETTER SHARP S, 0xDF
          ,0x00E0 --   LATIN SMALL LETTER A WITH GRAVE, 0xE0
          ,0x00E1 --   LATIN SMALL LETTER A WITH ACUTE, 0xE1
          ,0x00E2 --   LATIN SMALL LETTER A WITH CIRCUMFLEX, 0xE2
          ,0x0103 --   LATIN SMALL LETTER A WITH BREVE, 0xE3
          ,0x00E4 --   LATIN SMALL LETTER A WITH DIAERESIS, 0xE4
          ,0x0107 --   LATIN SMALL LETTER C WITH ACUTE, 0xE5
          ,0x00E6 --   LATIN SMALL LETTER AE, 0xE6
          ,0x00E7 --   LATIN SMALL LETTER C WITH CEDILLA, 0xE7
          ,0x00E8 --   LATIN SMALL LETTER E WITH GRAVE, 0xE8
          ,0x00E9 --   LATIN SMALL LETTER E WITH ACUTE, 0xE9
          ,0x00EA --   LATIN SMALL LETTER E WITH CIRCUMFLEX, 0xEA
          ,0x00EB --   LATIN SMALL LETTER E WITH DIAERESIS, 0xEB
          ,0x00EC --   LATIN SMALL LETTER I WITH GRAVE, 0xEC
          ,0x00ED --   LATIN SMALL LETTER I WITH ACUTE, 0xED
          ,0x00EE --   LATIN SMALL LETTER I WITH CIRCUMFLEX, 0xEE
          ,0x00EF --   LATIN SMALL LETTER I WITH DIAERESIS, 0xEF
          ,0x0111 --   LATIN SMALL LETTER D WITH STROKE, 0xF0
          ,0x0144 --   LATIN SMALL LETTER N WITH ACUTE, 0xF1
          ,0x00F2 --   LATIN SMALL LETTER O WITH GRAVE, 0xF2
          ,0x00F3 --   LATIN SMALL LETTER O WITH ACUTE, 0xF3
          ,0x00F4 --   LATIN SMALL LETTER O WITH CIRCUMFLEX, 0xF4
          ,0x0151 --   LATIN SMALL LETTER O WITH DOUBLE ACUTE, 0xF5
          ,0x00F6 --   LATIN SMALL LETTER O WITH DIAERESIS, 0xF6
          ,0x015B --   LATIN SMALL LETTER S WITH ACUTE, 0xF7
          ,0x0171 --   LATIN SMALL LETTER U WITH DOUBLE ACUTE, 0xF8
          ,0x00F9 --   LATIN SMALL LETTER U WITH GRAVE, 0xF9
          ,0x00FA --   LATIN SMALL LETTER U WITH ACUTE, 0xFA
          ,0x00FB --   LATIN SMALL LETTER U WITH CIRCUMFLEX, 0xFB
          ,0x00FC --   LATIN SMALL LETTER U WITH DIAERESIS, 0xFC
          ,0x0119 --   LATIN SMALL LETTER E WITH OGONEK, 0xFD
          ,0x021B --   LATIN SMALL LETTER T WITH COMMA BELOW, 0xFE
          ,0x00FF --   LATIN SMALL LETTER Y WITH DIAERESIS, 0xFF
          ]
