{-# OPTIONS -cpp -fffi -fglasgow-exts #-}

-- Module:    Data.ByteString.KOI8-R

-- Manipulate ByteStrings using Char operations, interpreting the strings
-- as Russian text in KOI8_R, a.k.a. IBM-878, a.k.a CP20866

module Data.ByteString.KOI8_R
#include "char_functions.hsi"

charset_name :: String
charset_name = "KOI8_R"

space, newline :: Word8
space = 0x20
newline = 0x0A

isSpaceWord8 :: Word8 -> Bool
isSpaceWord8 w = case w of 0x9A -> True -- NO-BREAK SPACE
                           _    -> isAsciiSpace w
{-# INLINE isSpaceWord8 #-}

#include "derive.hsi"

is_valid :: Char -> Bool
STRICT1(is_valid)
is_valid c = ord c < 0x80
          || (case bsearch char_idx 0x80 0xFF c of Just _  -> True
			                           Nothing -> False)
c2w :: Char -> Word8
STRICT1(c2w)
c2w c    = if ord c < 0x80 then c2w_raw c else c2w_d c

w2c :: Word8 -> Char
STRICT1(w2c)
w2c w    = if w < 0x80 then w2c_raw w else w2c_d w

charset :: [Int]
charset = [0..0x7F]   -- ASCII subset + controls
       ++ [0x2500  --	BOX DRAWINGS LIGHT HORIZONTAL, 0x80
          ,0x2502  --	BOX DRAWINGS LIGHT VERTICAL, 0x81
          ,0x250C  --	BOX DRAWINGS LIGHT DOWN AND RIGHT, 0x82
          ,0x2510  --	BOX DRAWINGS LIGHT DOWN AND LEFT, 0x83
          ,0x2514  --	BOX DRAWINGS LIGHT UP AND RIGHT, 0x84
          ,0x2518  --	BOX DRAWINGS LIGHT UP AND LEFT, 0x85
          ,0x251C  --	BOX DRAWINGS LIGHT VERTICAL AND RIGHT, 0x86
          ,0x2524  --	BOX DRAWINGS LIGHT VERTICAL AND LEFT, 0x87
          ,0x252C  --	BOX DRAWINGS LIGHT DOWN AND HORIZONTAL, 0x88
          ,0x2534  --	BOX DRAWINGS LIGHT UP AND HORIZONTAL, 0x89
          ,0x253C  --	BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL, 0x8A
          ,0x2580  --	UPPER HALF BLOCK, 0x8B
          ,0x2584  --	LOWER HALF BLOCK, 0x8C
          ,0x2588  --	FULL BLOCK, 0x8D
          ,0x258C  --	LEFT HALF BLOCK, 0x8E
          ,0x2590  --	RIGHT HALF BLOCK, 0x8F
          ,0x2591  --	LIGHT SHADE, 0x90
          ,0x2592  --	MEDIUM SHADE, 0x91
          ,0x2593  --	DARK SHADE, 0x92
          ,0x2320  --	TOP HALF INTEGRAL, 0x93
          ,0x25A0  --	BLACK SQUARE, 0x94
          ,0x2219  --	BULLET OPERATOR, 0x95
          ,0x221A  --	SQUARE ROOT, 0x96
          ,0x2248  --	ALMOST EQUAL TO, 0x97
          ,0x2264  --	LESS-THAN OR EQUAL TO, 0x98
          ,0x2265  --	GREATER-THAN OR EQUAL TO, 0x99
          ,0x00A0  --	NO-BREAK SPACE, 0x9A
          ,0x2321  --	BOTTOM HALF INTEGRAL, 0x9B
          ,0x00B0  --	DEGREE SIGN, 0x9C
          ,0x00B2  --	SUPERSCRIPT TWO, 0x9D
          ,0x00B7  --	MIDDLE DOT, 0x9E
          ,0x00F7  --	DIVISION SIGN, 0x9F
          ,0x2550  --	BOX DRAWINGS DOUBLE HORIZONTAL, 0xA0
          ,0x2551  --	BOX DRAWINGS DOUBLE VERTICAL, 0xA1
          ,0x2552  --	BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE, 0xA2
          ,0x0451  --	CYRILLIC SMALL LETTER IO, 0xA3
          ,0x2553  --	BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE, 0xA4
          ,0x2554  --	BOX DRAWINGS DOUBLE DOWN AND RIGHT, 0xA5
          ,0x2555  --	BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE, 0xA6
          ,0x2556  --	BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE, 0xA7
          ,0x2557  --	BOX DRAWINGS DOUBLE DOWN AND LEFT, 0xA8
          ,0x2558  --	BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE, 0xA9
          ,0x2559  --	BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE, 0xAA
          ,0x255A  --	BOX DRAWINGS DOUBLE UP AND RIGHT, 0xAB
          ,0x255B  --	BOX DRAWINGS UP SINGLE AND LEFT DOUBLE, 0xAC
          ,0x255C  --	BOX DRAWINGS UP DOUBLE AND LEFT SINGLE, 0xAD
          ,0x255D  --	BOX DRAWINGS DOUBLE UP AND LEFT, 0xAE
          ,0x255E  --	BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE, 0xAF
          ,0x255F  --	BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE, 0xB0
          ,0x2560  --	BOX DRAWINGS DOUBLE VERTICAL AND RIGHT, 0xB1
          ,0x2561  --	BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE, 0xB2
          ,0x0401  --	CYRILLIC CAPITAL LETTER IO, 0xB3
          ,0x2562  --	BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE, 0xB4
          ,0x2563  --	BOX DRAWINGS DOUBLE VERTICAL AND LEFT, 0xB5
          ,0x2564  --	BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE, 0xB6
          ,0x2565  --	BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE, 0xB7
          ,0x2566  --	BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL, 0xB8
          ,0x2567  --	BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE, 0xB9
          ,0x2568  --	BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE, 0xBA
          ,0x2569  --	BOX DRAWINGS DOUBLE UP AND HORIZONTAL, 0xBB
          ,0x256A  --	BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE, 0xBC
          ,0x256B  --	BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE, 0xBD
          ,0x256C  --	BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL, 0xBE
          ,0x00A9  --	COPYRIGHT SIGN, 0xBF
          ,0x044E  --	CYRILLIC SMALL LETTER YU, 0xC0
          ,0x0430  --	CYRILLIC SMALL LETTER A, 0xC1
          ,0x0431  --	CYRILLIC SMALL LETTER BE, 0xC2
          ,0x0446  --	CYRILLIC SMALL LETTER TSE, 0xC3
          ,0x0434  --	CYRILLIC SMALL LETTER DE, 0xC4
          ,0x0435  --	CYRILLIC SMALL LETTER IE, 0xC5
          ,0x0444  --	CYRILLIC SMALL LETTER EF, 0xC6
          ,0x0433  --	CYRILLIC SMALL LETTER GHE, 0xC7
          ,0x0445  --	CYRILLIC SMALL LETTER HA, 0xC8
          ,0x0438  --	CYRILLIC SMALL LETTER I, 0xC9
          ,0x0439  --	CYRILLIC SMALL LETTER SHORT I, 0xCA
          ,0x043A  --	CYRILLIC SMALL LETTER KA, 0xCB
          ,0x043B  --	CYRILLIC SMALL LETTER EL, 0xCC
          ,0x043C  --	CYRILLIC SMALL LETTER EM, 0xCD
          ,0x043D  --	CYRILLIC SMALL LETTER EN, 0xCE
          ,0x043E  --	CYRILLIC SMALL LETTER O, 0xCF
          ,0x043F  --	CYRILLIC SMALL LETTER PE, 0xD0
          ,0x044F  --	CYRILLIC SMALL LETTER YA, 0xD1
          ,0x0440  --	CYRILLIC SMALL LETTER ER, 0xD2
          ,0x0441  --	CYRILLIC SMALL LETTER ES, 0xD3
          ,0x0442  --	CYRILLIC SMALL LETTER TE, 0xD4
          ,0x0443  --	CYRILLIC SMALL LETTER U, 0xD5
          ,0x0436  --	CYRILLIC SMALL LETTER ZHE, 0xD6
          ,0x0432  --	CYRILLIC SMALL LETTER VE, 0xD7
          ,0x044C  --	CYRILLIC SMALL LETTER SOFT SIGN, 0xD8
          ,0x044B  --	CYRILLIC SMALL LETTER YERU, 0xD9
          ,0x0437  --	CYRILLIC SMALL LETTER ZE, 0xDA
          ,0x0448  --	CYRILLIC SMALL LETTER SHA, 0xDB
          ,0x044D  --	CYRILLIC SMALL LETTER E, 0xDC
          ,0x0449  --	CYRILLIC SMALL LETTER SHCHA, 0xDD
          ,0x0447  --	CYRILLIC SMALL LETTER CHE, 0xDE
          ,0x044A  --	CYRILLIC SMALL LETTER HARD SIGN, 0xDF
          ,0x042E  --	CYRILLIC CAPITAL LETTER YU, 0xE0
          ,0x0410  --	CYRILLIC CAPITAL LETTER A, 0xE1
          ,0x0411  --	CYRILLIC CAPITAL LETTER BE, 0xE2
          ,0x0426  --	CYRILLIC CAPITAL LETTER TSE, 0xE3
          ,0x0414  --	CYRILLIC CAPITAL LETTER DE, 0xE4
          ,0x0415  --	CYRILLIC CAPITAL LETTER IE, 0xE5
          ,0x0424  --	CYRILLIC CAPITAL LETTER EF, 0xE6
          ,0x0413  --	CYRILLIC CAPITAL LETTER GHE, 0xE7
          ,0x0425  --	CYRILLIC CAPITAL LETTER HA, 0xE8
          ,0x0418  --	CYRILLIC CAPITAL LETTER I, 0xE9
          ,0x0419  --	CYRILLIC CAPITAL LETTER SHORT I, 0xEA
          ,0x041A  --	CYRILLIC CAPITAL LETTER KA, 0xEB
          ,0x041B  --	CYRILLIC CAPITAL LETTER EL, 0xEC
          ,0x041C  --	CYRILLIC CAPITAL LETTER EM, 0xED
          ,0x041D  --	CYRILLIC CAPITAL LETTER EN, 0xEE
          ,0x041E  --	CYRILLIC CAPITAL LETTER O, 0xEF
          ,0x041F  --	CYRILLIC CAPITAL LETTER PE, 0xF0
          ,0x042F  --	CYRILLIC CAPITAL LETTER YA, 0xF1
          ,0x0420  --	CYRILLIC CAPITAL LETTER ER, 0xF2
          ,0x0421  --	CYRILLIC CAPITAL LETTER ES, 0xF3
          ,0x0422  --	CYRILLIC CAPITAL LETTER TE, 0xF4
          ,0x0423  --	CYRILLIC CAPITAL LETTER U, 0xF5
          ,0x0416  --	CYRILLIC CAPITAL LETTER ZHE, 0xF6
          ,0x0412  --	CYRILLIC CAPITAL LETTER VE, 0xF7
          ,0x042C  --	CYRILLIC CAPITAL LETTER SOFT SIGN, 0xF8
          ,0x042B  --	CYRILLIC CAPITAL LETTER YERU, 0xF9
          ,0x0417  --	CYRILLIC CAPITAL LETTER ZE, 0xFA
          ,0x0428  --	CYRILLIC CAPITAL LETTER SHA, 0xFB
          ,0x042D  --	CYRILLIC CAPITAL LETTER E, 0xFC
          ,0x0429  --	CYRILLIC CAPITAL LETTER SHCHA, 0xFD
          ,0x0427  --	CYRILLIC CAPITAL LETTER CHE, 0xFE
          ,0x042A  --	CYRILLIC CAPITAL LETTER HARD SIGN, 0xFF
          ]
