{-# OPTIONS -cpp -fffi -fglasgow-exts #-}

-- Module:    Data.ByteString.Win1252

-- Manipulate ByteStrings using Char operations, interpreting the strings
-- as Windows-1252.  This is an extension to ISO_8859-1 (Latin1), and
-- is most notable for stuffing funny quotes in the range 0x80-09F

module Data.ByteString.Win1252
#include "char_functions.hsi"

charset_name :: String
charset_name = "Win1252"

space, newline :: Word8
space = 0x20
newline = 0x0A

isSpaceWord8 :: Word8 -> Bool
isSpaceWord8 w = case w of 0xA0 -> True -- NO-BREAK SPACE (ISO 8859-1)
                           _   -> isAsciiSpace w
{-# INLINE isSpaceWord8 #-}

#include "derive.hsi"

is_direct w = (w >= 0 && w <= 0x7F) 
           || (w >= 0xA0 && w <= 0xFF)

is_valid c = is_direct (fromIntegral $ ord c)
          || (case bsearch char_idx 224 255 c of Just _  -> True
			                         Nothing -> False)
c2w c    = if is_direct (fromIntegral $ ord c) then c2w_raw c else c2w_d c
w2c w    = if is_direct w then w2c_raw w else w2c_d w

charset :: [Int]
charset = [0..0x7F]++[ -- ASCII subset
           0x20AC  -- EURO SIGN
          ,0xF100  -- unused
          ,0x201A  -- SINGLE LOW-9 QUOTATION MARK
          ,0x0192  -- LATIN SMALL LETTER F WITH HOOK
          ,0x201E  -- DOUBLE LOW-9 QUOTATION MARK
          ,0x2026  -- HORIZONTAL ELLIPSIS
          ,0x2020  -- DAGGER
          ,0x2021  -- DOUBLE DAGGER
          ,0x02C6  -- MODIFIER LETTER CIRCUMFLEX ACCENT
          ,0x2030  -- PER MILLE SIGN
          ,0x0160  -- LATIN CAPITAL LETTER S WITH CARON
          ,0x2039  -- SINGLE LEFT-POINTING ANGLE QUOTATION MARK
          ,0x0152  -- LATIN CAPITAL LIGATURE OE
          ,0xF101  -- unused
          ,0x017D  -- LATIN CAPITAL LETTER Z WITH CARON
          ,0xF102, 0xF103  -- unused
          ,0x2018  -- LEFT SINGLE QUOTATION MARK
          ,0x2019  -- RIGHT SINGLE QUOTATION MARK
          ,0x201C  -- LEFT DOUBLE QUOTATION MARK
          ,0x201D  -- RIGHT DOUBLE QUOTATION MARK
          ,0x2022  -- BULLET
          ,0x2013  -- EN DASH
          ,0x2014  -- EM DASH
          ,0x02DC  -- SMALL TILDE
          ,0x2122  -- TRADE MARK SIGN
          ,0x0161  -- LATIN SMALL LETTER S WITH CARON
          ,0x203A  -- SINGLE RIGHT-POINTING ANGLE QUOTATION MARK
          ,0x0153  -- LATIN SMALL LIGATURE OE
          ,0xF104  -- unused
          ,0x017E  -- LATIN SMALL LETTER Z WITH CARON
          ,0x0178  -- LATIN CAPITAL LETTER Y WITH DIAERESIS
          ]++[0xA0..0xFF] -- ISO_8859-1 subset




